#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/Label.h>
#include "LinkView.h"
#include "LinkText.h"
#include "PcBar.h"

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "moviedb.h"
#include "dbutils.h"
#include "xregal.h"
#include "popup.h"
#include "tree.h"
#include "keyword.h"
#include "printmovies.h"
#include "lookuptitle.h"

/* Function prototypes */
static void searchFromKeywordList(Widget, XEvent*, String*, Cardinal*);

Boolean ks_popped_up=False;
int keyword_stage=0;
char keyword_searchtext[ MXLINELEN ] ;

static Widget keyword_list=NULL, 
		keyword_shell=NULL,
		keywordmenu,
		keywordmenubutton;
static Widget keyword_form, keyword_label, keyword_search, keyword_cancel, keyword_view;

static Tree keywords;
static char **keyword_array;

static String myDouble = "<Btn1Up>(2):   searchFromKeywordList()\n";
static XtTranslations doubleTransl=NULL;

static XtActionsRec actionTable[] = {
	{"searchFromKeywordList",searchFromKeywordList}
	};

typedef struct {
	char *text;
	int value;
	} KeywordItem;

KeywordItem keyword_menu[] = {
	{"  1 item", 1},
	{"  2 items", 2},
	{"  5 items", 5},
	{" 10 items", 10},
	{" 20 items", 20},
	{" 50 items", 50},
	{"100 items", 100},
	{"200 items", 200},
	{"500 items", 500}
	};
#define KEYWORD_MENU_ITEMS 9
static Widget keyword_menu_widget[KEYWORD_MENU_ITEMS];


Boolean doKeywordSearch(XtPointer client_data) {
	static FILE  *dbFp = NULL;
	static FILE  *titleKeyFp = NULL;
	static FILE  *titleIndexFp = NULL;
	TitleID titleKey ;
	int len ;
	char line [MXLINELEN];
	AttributeID attrKey;
	static TitleRec *head;
	static TitleRec *tail;
	static int count;
	static long upper;
	static int lastpcdone=0;
	int final_message;
	int pcdone;
	char buf[MAXPATHLEN];

	if (keyword_stage==0) {
		count=0;
		dbFp = openFile (KYWDDB);
		titleKeyFp = openFile(TITLEKEY);
		titleIndexFp = openFile(TITLEIDX);
		fseek(dbFp, 0, SEEK_END);
		upper=ftell(dbFp);
		rewind(dbFp);
		keyword_stage++;
		return False;
		}

	if (keyword_stage == 1) {
		while (!feof(dbFp)) { 

			/* update display */
			pcdone=ftell(dbFp)*100/upper;
			if (pcdone != lastpcdone) {
				lastpcdone=pcdone;
				if (app_data.showprogress) {
					PcBarSetPercentage(pcbar, pcdone);
					sprintf(buf, "Matches on '%s': %d", keyword_searchtext, count);
					SetTitle(buf);
					}
				return False;
				}

		    titleKey = getTitle ( dbFp ) ;
	    	len = getByte ( dbFp ) ;
	    	fread(line, 1, len, dbFp ) ;
			line [len] = '\0' ;
    		attrKey = getAttr ( dbFp ) ;
			if (strcmp(line, keyword_searchtext)==0) {
				if (count == 0) {
					head = newTitleRec();
					head->title = mapTitleKeyToText(titleKey, titleKeyFp, titleIndexFp);
					head->titleKey = titleKey;
					tail=head;
					}
				else {
					tail->next = newTitleRec();
					tail = tail->next;
					tail->title = mapTitleKeyToText(titleKey, titleKeyFp, titleIndexFp);
					tail->titleKey = titleKey;
					}	
				count++;	
				}
			} 

		fclose(dbFp);
		fclose(titleKeyFp);
		fclose(titleIndexFp);
		keyword_stage++;
		if (app_data.showprogress) return False;
		}

	PcBarSetPercentage(pcbar, 0);

	final_message=SEARCH_OK;

	sprintf(line, "Keyword: %s", keyword_searchtext);
	if (count<200)
		addAka(head);
	displayListOfMovies(head, line, final_message);
	freeTitles(head);
	SetBusy(viewer, False);
	keyword_stage=0;
	search_in_progress=False;
	return True;
	}



static void Search(Widget w, XtPointer client_data, XtPointer call_data) {
	int index;
	char *cp;

	index=XawListShowCurrent(keyword_list)->list_index;
	if (index ==  XAW_LIST_NONE)
		return;

	XtPopdown(keyword_shell);
	ks_popped_up=False;	
	enableSearch();
	SetBusy(viewer, True);
	XFlush(XtDisplay(topLevel));

	strcpy(keyword_searchtext, keyword_array[index]);
	cp=strchr(keyword_searchtext, ' ');
	if (cp!=NULL)
		*cp='\0';

	search_in_progress=True;
	XtAppAddWorkProc(app_context, doKeywordSearch, NULL);
	}

void searchFromKeywordList(Widget w, XEvent *ev, String *x, Cardinal *y) {
	Search(w, NULL, NULL);
	}

static void Cancel(Widget w, XtPointer client_data, XtPointer call_data) {
	XtPopdown(keyword_shell);
	ks_popped_up=False;
	enableSearch();
	}

static void KeywordMenu(Widget w, XtPointer client_data, XtPointer garbage) {
	int item = (int) client_data;
	int i;

	/* Free the existing keyword array. */
	i=0;
	while (keyword_array[i] != NULL) 
		free(keyword_array[i++]);
	free(keyword_array);

	/* create the new keyword array */
	keyword_array=TreeToArray(&keywords, keyword_menu[item].value);
	XawListChange(keyword_list, keyword_array, 0, 0,True);
	if (item < 2) 
		XtVaSetValues(keyword_list, XtNdefaultColumns, 4, NULL);
	else
		XtVaSetValues(keyword_list, XtNdefaultColumns, 2, NULL);
	XtVaSetValues(keywordmenubutton, XtNlabel, keyword_menu[item].text, NULL);
	}



void CreateKeywordDialog(Widget parent) {
	static String myreturn = "#override <Key>Return:   set() notify() unset()\n";
	static String myescape = "#override <Key>Escape:   set() notify() unset()\n";
	XtAccelerators escaccel;
	XtAccelerators accel;
	Dimension spacing;
	XFontStruct *font;
	int i;

	accel = XtParseAcceleratorTable(myreturn);
	escaccel = XtParseAcceleratorTable(myescape);

	keyword_shell=XtVaCreatePopupShell("keyword_shell",
                transientShellWidgetClass,  parent,
                XtNallowShellResize, True, NULL);

	keyword_form=XtVaCreateManagedWidget("keyword_form",formWidgetClass,
				keyword_shell, NULL);
	keyword_label=XtVaCreateManagedWidget("keyword_label", labelWidgetClass,
				keyword_form, 
				XtNborderWidth, 0,
				XtNbottom, XtChainTop,
				XtNleft, XtChainLeft,
				XtNright, XtChainLeft,
				NULL);
	keywordmenubutton = XtVaCreateManagedWidget("keywordmenubutton", 
				menuButtonWidgetClass, keyword_form, 
		        XtNmenuName, "keywordmenu",
				XtNfromHoriz, keyword_label,
				XtNbottom, XtChainTop,
				XtNleft, XtChainLeft,
				XtNright, XtChainLeft,
				XtNlabel, keyword_menu[app_data.keywordInitial].text,
				NULL);
	keywordmenu = XtVaCreatePopupShell("keywordmenu",simpleMenuWidgetClass, 
		keywordmenubutton,
        XtNborderWidth, 2,
        NULL);

    for (i=0; i<KEYWORD_MENU_ITEMS; i++) {
        String item = keyword_menu[i].text;
        keyword_menu_widget[i] = XtVaCreateManagedWidget(item, smeBSBObjectClass, keywordmenu, 
                XtNjustify, XtJustifyLeft,
                XtNlabel, item,
                NULL);

        XtAddCallback(keyword_menu_widget[i], XtNcallback, KeywordMenu, (XtPointer) i);
        }

	keyword_view=XtVaCreateManagedWidget("keyword_view",linkViewWidgetClass,
				keyword_form,
		        XtNborderWidth, 2,
				XtNfromVert, keyword_label,
		        XtNallowVert, True,
	        	XtNallowHoriz, True,
				XtNbottom, XtChainBottom,
		        NULL);


  keyword_list=XtVaCreateManagedWidget("keyword_list",listWidgetClass,keyword_view,
        XtNlist, NULL,
        NULL);

  XtVaGetValues(keyword_list, XtNrowSpacing, &spacing, XtNfont, &font, NULL);
  spacing +=  font->max_bounds.ascent + font->max_bounds.descent;
  XtVaSetValues(keyword_view, XtNbaselineskip, spacing, NULL);

  keyword_search=XtVaCreateManagedWidget("keyword_search",commandWidgetClass, keyword_form,
        XtNfromVert, keyword_view,
		XtNlabel, "Search",
        XtNaccelerators, accel,
		XtNleft, XtChainLeft,
		XtNright, XtChainLeft,
		XtNtop, XtChainBottom,
		XtNbottom, XtChainBottom,
		NULL);
  keyword_cancel=XtVaCreateManagedWidget("keyword_cancel",commandWidgetClass,keyword_form,
        XtNfromVert, keyword_view,
		XtNfromHoriz, keyword_search,
		XtNlabel, "Cancel",
        XtNaccelerators, escaccel,
		XtNleft, XtChainLeft,
		XtNright, XtChainLeft,
		XtNtop, XtChainBottom,
		XtNbottom, XtChainBottom,
		NULL);

  XtAddCallback(keyword_cancel, XtNcallback, Cancel, 0);
  XtAddCallback(keyword_search, XtNcallback, Search, 0);
  XtSetKeyboardFocus(keyword_form, keyword_view);
  XtInstallAccelerators(keyword_view, keyword_cancel);
  XtInstallAccelerators(keyword_view, keyword_search);

	if (!doubleTransl) {
		XtAppAddActions(app_context, actionTable, 1);
		doubleTransl = XtParseTranslationTable(myDouble);
		}
	XtOverrideTranslations(keyword_list, doubleTransl);
	}


	


void read_keywords(void) {
	FILE  *dbFp = NULL;
	TitleID titleKey ;
	int len ;
	char line [ MXLINELEN ] ;
	AttributeID attrKey;

	dbFp = openFile (KYWDDB);
	TreeInitialize(&keywords);
	while (!feof(dbFp)) { 
	    titleKey = getTitle ( dbFp ) ;
	    len = getByte ( dbFp ) ;
    	fread(line, 1, len, dbFp ) ;
		line [len] = '\0' ;
    	attrKey = getAttr ( dbFp ) ;
		TreeInsert(&keywords, line);
		} 
	keyword_array=TreeToArray(&keywords, keyword_menu[app_data.keywordInitial].value);
	fclose(dbFp);
	}



void PopupKeyword(void) {
	if (keyword_shell==NULL) {
		SetBusy(viewer, True);
		XFlush(XtDisplay(topLevel));
		CreateKeywordDialog(topLevel);
		read_keywords();
		XawListChange(keyword_list, keyword_array, 0, 0,True);
		SetBusy(viewer, False);
		}

	disableSearch();
	SetPopupPosition(keyword_shell, topLevel);
	XtPopup(keyword_shell, XtGrabExclusive);
	XDefineCursor(XtDisplay(topLevel), XtWindow(keyword_form), unBusyCursor);
	ks_popped_up=True;
	XawListChange(keyword_list, keyword_array, 0, 0,True);
	}

